#include "orbit.hpp"

#include "astro/stateVectorIndices.hpp"
#include "astro/orbitalElementConversions.hpp"

Orbit::Orbit(int vertexCount) :
    _keplerianElements(std::vector<float>(6))
{
    _keplerianElements[astro::semiMajorAxisIndex] = .75;
    _keplerianElements[astro::eccentricityIndex] = .1;
    _keplerianElements[astro::inclinationIndex] = _pi / 2.0 + 0.1;
    _keplerianElements[astro::argumentOfPeriapsisIndex] = 0;
    _keplerianElements[astro::longitudeOfAscendingNodeIndex] = 0;

    for (int i = 0; i < vertexCount; i++)
    {
        float t = (float)i / (float)vertexCount;
        glm::vec3 pos = getPosition(t);

        _vertices.push_back(pos.x);
        _vertices.push_back(pos.y);
        _vertices.push_back(pos.z);
    }

    glGenVertexArrays(1, &_vao);
    glGenBuffers(1, &_vbo);

    glBindVertexArray(_vao);

    glBindBuffer(GL_ARRAY_BUFFER, _vbo);
    size_t vboBufferSize = _vertices.size() * sizeof(float);
    glBufferData(GL_ARRAY_BUFFER, vboBufferSize, &_vertices[0], GL_STATIC_DRAW);

    glEnableVertexAttribArray(0);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);

    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindVertexArray(0);
}

// Interpolate a position around the orbit.
// t is in range 0..1 and wraps.
glm::vec3 Orbit::getPosition(float t)
{
    float a = t * 2.0 * _pi;

    std::vector<float> kepler(_keplerianElements);
    kepler[astro::trueAnomalyIndex] = a;

    std::vector<float> cartesian = astro::convertKeplerianToCartesianElements(kepler, 1.0);
    return glm::vec3(
        cartesian[astro::xPositionIndex],
        cartesian[astro::yPositionIndex],
        cartesian[astro::zPositionIndex]);
}

void Orbit::render()
{
    glBindVertexArray(_vao);
    glBindBuffer(GL_ARRAY_BUFFER, _vbo);

    glDrawArrays(GL_LINE_LOOP, 0, _vertices.size() / 3);
}

Orbit::~Orbit()
{
    glDeleteVertexArrays(1, &_vao);
    glDeleteBuffers(1, &_vbo);
}

